@@ -1,4 +1,3 @@
-{-# LANGUAGE TupleSections #-}
 -- incomplete patterns in 'fromDistinct{Asc,Desc}List'
 {-# OPTIONS_GHC -Wno-incomplete-uni-patterns #-}
 
@@ -7,6 +6,9 @@ module Mini.Data.Map (
   -- * Type
   Map,
 
+  -- * Primitive Recursion
+  map,
+
   -- * Construction
   empty,
   singleton,
@@ -217,23 +219,42 @@ instance (Ord k) => Monoid (Map k a) where
  - Primitive recursion
  -}
 
--- | Primitive recursion on maps
+-- | Primitive recursion on maps (internally structured as trees)
 map
   :: b
-  -- ^ Empty bin
+  -- ^ Value yielded in case of empty node
   -> (Map k a -> k -> a -> Map k a -> b -> b -> b)
-  -- ^ Left-heavy bin
-  -> (Map k a -> k -> a -> Map k a -> b -> b -> b)
-  -- ^ Balanced bin
-  -> (Map k a -> k -> a -> Map k a -> b -> b -> b)
-  -- ^ Right-heavy bin
+  -- ^ Function applied in case of non-empty node:
+  -- left child, key, value, right child, left recursion, right recursion
+  -- (keys are lesser to the left, greater to the right)
   -> Map k a
-  -- ^ Map
+  -- ^ Object of the case analysis
   -> b
-map e _ _ _ E = e
-map e f g h (L l k a r) = f l k a r (map e f g h l) (map e f g h r)
-map e f g h (B l k a r) = g l k a r (map e f g h l) (map e f g h r)
-map e f g h (R l k a r) = h l k a r (map e f g h l) (map e f g h r)
+map e f = map' e f f f
+
+-- Primitive recursion on maps
+map'
+  :: b
+  -- ^ Value yielded in case of empty node
+  -> (Map k a -> k -> a -> Map k a -> b -> b -> b)
+  -- ^ Function applied in case of left-heavy node:
+  -- left child, key, value, right child, left recursion, right recursion
+  -- (keys are lesser to the left, greater to the right)
+  -> (Map k a -> k -> a -> Map k a -> b -> b -> b)
+  -- ^ Function applied in case of balanced node:
+  -- left child, key, value, right child, left recursion, right recursion
+  -- (keys are lesser to the left, greater to the right)
+  -> (Map k a -> k -> a -> Map k a -> b -> b -> b)
+  -- ^ Function applied in case of right-heavy node:
+  -- left child, key, value, right child, left recursion, right recursion
+  -- (keys are lesser to the left, greater to the right)
+  -> Map k a
+  -- ^ Object of the case analysis
+  -> b
+map' e _ _ _ E = e
+map' e f g h (L l k a r) = f l k a r (map' e f g h l) (map' e f g h r)
+map' e f g h (B l k a r) = g l k a r (map' e f g h l) (map' e f g h r)
+map' e f g h (R l k a r) = h l k a r (map' e f g h l) (map' e f g h r)
 
 {-
  - Construction
@@ -315,7 +336,7 @@ fromDistinctDescList = go <*> power
 
 -- | /O(n log m)/ Compose the keys of one set with the values of another
 compose :: (Ord b) => Map b c -> Map a b -> Map a c
-compose t1 t2 = map empty go go go t1
+compose t1 t2 = map' empty go go go t1
  where
   go _ _ _ _ _ _ =
     fromDistinctAscList $
@@ -326,7 +347,7 @@ compose t1 t2 = map empty go go go t1
 
 -- | /O(m log n)/ Subtract a map by another via key matching
 difference :: (Ord k) => Map k a -> Map k b -> Map k a
-difference t1 t2 = map empty go go go t1
+difference t1 t2 = map' empty go go go t1
  where
   go _ _ _ _ _ _ = foldrWithKey (\k _ b -> delete k b) t1 t2
 
@@ -338,7 +359,7 @@ differenceWith = differenceWithKey . const
 -- | /O(m log n)/ Subtract a map by another, updating bins of matching keys
 differenceWithKey
   :: (Ord k) => (k -> a -> b -> Maybe a) -> Map k a -> Map k b -> Map k a
-differenceWithKey f t1 t2 = map empty go go go t1
+differenceWithKey f t1 t2 = map' empty go go go t1
  where
   go _ _ _ _ _ _ =
     foldrWithKey
@@ -367,7 +388,7 @@ intersectionWith = intersectionWithKey . const
 -- | /O(n log m)/ Intersect a map with another by key matching, combining values
 intersectionWithKey
   :: (Ord k) => (k -> a -> b -> c) -> Map k a -> Map k b -> Map k c
-intersectionWithKey f t1 t2 = map empty go go go t2
+intersectionWithKey f t1 t2 = map' empty go go go t2
  where
   go _ _ _ _ _ _ =
     fromDistinctAscList $
@@ -386,7 +407,7 @@ unionWith = unionWithKey . const
 
 -- | /O(m log n)/ Unite a map with another, combining values of matching keys
 unionWithKey :: (Ord k) => (k -> a -> a -> a) -> Map k a -> Map k a -> Map k a
-unionWithKey f t1 t2 = map t1 go go go t2
+unionWithKey f t1 t2 = map' t1 go go go t2
  where
   go _ _ _ _ _ _ = foldrWithKey (insertWithKey f) t2 t1
 
@@ -420,13 +441,13 @@ toDescList = foldlWithKey (\b k a -> (k, a) : b) []
 
 -- | /O(n)/ Reduce a map with a left-associative operation and an accumulator
 foldlWithKey :: (b -> k -> a -> b) -> b -> Map k a -> b
-foldlWithKey f b = map b go go go
+foldlWithKey f b = map' b go go go
  where
   go _ k a r recl _ = foldlWithKey f (f recl k a) r
 
 -- | /O(n)/ Reduce a map with a right-associative operation and an accumulator
 foldrWithKey :: (k -> a -> b -> b) -> b -> Map k a -> b
-foldrWithKey f b = map b go go go
+foldrWithKey f b = map' b go go go
  where
   go l k a _ _ recr = foldrWithKey f (f k a recr) l
 
@@ -440,7 +461,7 @@ adjust = adjustWithKey . const
 
 -- | /O(log n)/ Adjust with an operation the value of a key in a map
 adjustWithKey :: (Ord k) => (k -> a -> a) -> k -> Map k a -> Map k a
-adjustWithKey f k0 = map E (go L) (go B) (go R)
+adjustWithKey f k0 = map' E (go L) (go B) (go R)
  where
   go c l k a r recl recr = case compare k0 k of
     LT -> c recl k a r
@@ -453,9 +474,9 @@ adjustMax = adjustMaxWithKey . const
 
 -- | /O(log n)/ Adjust with an operation the value of the maximum key in a map
 adjustMaxWithKey :: (k -> a -> a) -> Map k a -> Map k a
-adjustMaxWithKey f = map E (go L) (go B) (go R)
+adjustMaxWithKey f = map' E (go L) (go B) (go R)
  where
-  go c l k a r _ recr = map (c l k (f k a) r) go' go' go' r
+  go c l k a r _ recr = map' (c l k (f k a) r) go' go' go' r
    where
     go' _ _ _ _ _ _ = c l k a recr
 
@@ -465,9 +486,9 @@ adjustMin = adjustMinWithKey . const
 
 -- | /O(log n)/ Adjust with an operation the value of the minimum key in a map
 adjustMinWithKey :: (k -> a -> a) -> Map k a -> Map k a
-adjustMinWithKey f = map E (go L) (go B) (go R)
+adjustMinWithKey f = map' E (go L) (go B) (go R)
  where
-  go c l k a r recl _ = map (c l k (f k a) r) go' go' go' l
+  go c l k a r recl _ = map' (c l k (f k a) r) go' go' go' l
    where
     go' _ _ _ _ _ _ = c recl k a r
 
@@ -593,7 +614,7 @@ splitMin t = ((,) <*> flip delete' t . fst) <$> lookupMin t
 
 -- | /O(m log n)/ Check whether two maps have no keys in common
 disjoint :: (Ord k) => Map k a -> Map k a -> Bool
-disjoint t1 t2 = map True go go go t1
+disjoint t1 t2 = map' True go go go t1
  where
   go _ _ _ _ _ _ = not $ foldrWithKey (\k _ b -> k `member` t1 || b) False t2
 
@@ -603,7 +624,7 @@ isSubmapOf = isSubmapOfBy (==)
 
 -- | /O(n log m)/ Check if the bins of one map exist in the other by combination
 isSubmapOfBy :: (Ord k) => (a -> b -> Bool) -> Map k a -> Map k b -> Bool
-isSubmapOfBy p t1 t2 = map (null t1) go go go t2
+isSubmapOfBy p t1 t2 = map' (null t1) go go go t2
  where
   go _ _ _ _ _ _ =
     foldrWithKey
@@ -613,7 +634,7 @@ isSubmapOfBy p t1 t2 = map (null t1) go go go t2
 
 -- | /O(log n)/ Fetch the value of a key in a map, or 'Nothing' if absent
 lookup :: (Ord k) => k -> Map k a -> Maybe a
-lookup k = map Nothing go go go
+lookup k = map' Nothing go go go
  where
   go _ k' a _ recl recr = case compare k k' of
     LT -> recl
@@ -622,7 +643,7 @@ lookup k = map Nothing go go go
 
 -- | /O(log n)/ Fetch the least bin greater than or equal to a key
 lookupGE :: (Ord k) => k -> Map k a -> Maybe (k, a)
-lookupGE k0 = map Nothing go go go
+lookupGE k0 = map' Nothing go go go
  where
   go _ k a _ recl recr = case compare k k0 of
     LT -> recr
@@ -631,7 +652,7 @@ lookupGE k0 = map Nothing go go go
 
 -- | /O(log n)/ Fetch the least bin strictly greater than a key
 lookupGT :: (Ord k) => k -> Map k a -> Maybe (k, a)
-lookupGT k0 = map Nothing go go go
+lookupGT k0 = map' Nothing go go go
  where
   go _ k a _ recl recr = case compare k k0 of
     LT -> recr
@@ -640,7 +661,7 @@ lookupGT k0 = map Nothing go go go
 
 -- | /O(log n)/ Fetch the greatest bin less than or equal to a key
 lookupLE :: (Ord k) => k -> Map k a -> Maybe (k, a)
-lookupLE k0 = map Nothing go go go
+lookupLE k0 = map' Nothing go go go
  where
   go _ k a _ recl recr = case compare k k0 of
     LT -> recr <|> Just (k, a)
@@ -649,7 +670,7 @@ lookupLE k0 = map Nothing go go go
 
 -- | /O(log n)/ Fetch the greatest bin strictly less than a key
 lookupLT :: (Ord k) => k -> Map k a -> Maybe (k, a)
-lookupLT k0 = map Nothing go go go
+lookupLT k0 = map' Nothing go go go
  where
   go _ k a _ recl recr = case compare k k0 of
     LT -> recr <|> Just (k, a)
@@ -658,23 +679,23 @@ lookupLT k0 = map Nothing go go go
 
 -- | /O(log n)/ Fetch the bin with the maximum key, or 'Nothing' if empty
 lookupMax :: Map k a -> Maybe (k, a)
-lookupMax = map Nothing go go go
+lookupMax = map' Nothing go go go
  where
-  go _ k a r _ recr = map (Just (k, a)) go' go' go' r
+  go _ k a r _ recr = map' (Just (k, a)) go' go' go' r
    where
     go' _ _ _ _ _ _ = recr
 
 -- | /O(log n)/ Fetch the bin with the minimum key, or 'Nothing' if empty
 lookupMin :: Map k a -> Maybe (k, a)
-lookupMin = map Nothing go go go
+lookupMin = map' Nothing go go go
  where
-  go l k a _ recl _ = map (Just (k, a)) go' go' go' l
+  go l k a _ recl _ = map' (Just (k, a)) go' go' go' l
    where
     go' _ _ _ _ _ _ = recl
 
 -- | /O(log n)/ Check whether a key is in a map
 member :: (Ord k) => k -> Map k a -> Bool
-member k0 = map False go go go
+member k0 = map' False go go go
  where
   go _ k _ _ recl recr = case compare k0 k of
     LT -> recl
@@ -683,11 +704,11 @@ member k0 = map False go go go
 
 -- | /O(1)/ Check whether a map is empty
 null :: Map k a -> Bool
-null = map True go go go where go _ _ _ _ _ _ = False
+null = map' True go go go where go _ _ _ _ _ _ = False
 
 -- | /O(n)/ Get the size of a map
 size :: Map k a -> Int
-size = map 0 go go go where go _ _ _ _ recl recr = 1 + recl + recr
+size = map' 0 go go go where go _ _ _ _ recl recr = 1 + recl + recr
 
 {-
  - Traversal
@@ -695,13 +716,13 @@ size = map 0 go go go where go _ _ _ _ recl recr = 1 + recl + recr
 
 -- | /O(n)/ Apply an operation across a map, transforming its values
 fmapWithKey :: (k -> a -> b) -> Map k a -> Map k b
-fmapWithKey f = map E (go L) (go B) (go R)
+fmapWithKey f = map' E (go L) (go B) (go R)
  where
   go c _ k a _ recl = c recl k (f k a)
 
 -- | /O(n)/ Lift a map with a lifting operation on keys and values
 traverseWithKey :: (Applicative f) => (k -> a -> f b) -> Map k a -> f (Map k b)
-traverseWithKey f = map (pure E) (go L) (go B) (go R)
+traverseWithKey f = map' (pure E) (go L) (go B) (go R)
  where
   go c _ k a _ recl recr = c <$> recl <*> pure k <*> f k a <*> recr
 
@@ -714,19 +735,19 @@ valid :: (Ord k) => Map k a -> Bool
 valid = liftA2 (&&) balanced ordered
  where
   balanced =
-    map
+    map'
       True
       (\l _ _ r recl recr -> levels l - levels r == 1 && recl && recr)
       (\l _ _ r recl recr -> levels l - levels r == 0 && recl && recr)
       (\l _ _ r recl recr -> levels r - levels l == 1 && recl && recr)
-  levels = map 0 go go go
+  levels = map' 0 go go go
    where
     go _ _ _ _ recl recr = 1 + max recl recr :: Int
-  ordered = map True go go go
+  ordered = map' True go go go
    where
     go l k _ r recl recr =
-      map True lt lt lt l
-        && map True gt gt gt r
+      map' True lt lt lt l
+        && map' True gt gt gt r
      where
       lt _ lk _ _ _ _ = lk < k && recl && recr
       gt _ rk _ _ _ _ = rk > k && recl && recr
@@ -769,7 +790,7 @@ power as = until (> length as) (* 2) 2 `div` 2
 -- O(log n) Delete a key from a map without checking for membership
 delete' :: (Ord k) => k -> Map k a -> Map k a
 delete' k0 =
-  map
+  map'
     (error "Map.delete: L0")
     ( \l k a r _ _ ->
         case compare k0 k of
@@ -791,7 +812,7 @@ delete' k0 =
     )
  where
   deleteRl l k a r =
-    map
+    map'
       (error "Map.delete: L1")
       ( \ll lk la lr _ _ ->
           case compare k0 lk of
@@ -813,7 +834,7 @@ delete' k0 =
       )
       l
   deleteRr l k a =
-    map
+    map'
       (error "Map.delete: L2")
       ( \rl rk ra rr _ _ ->
           case compare k0 rk of
@@ -834,7 +855,7 @@ delete' k0 =
             GT -> checkRightR l k a (deleteRr rl rk ra rr)
       )
   deleteBl l k a r =
-    map
+    map'
       (error "Map.delete: L3")
       ( \ll lk la lr _ _ ->
           case compare k0 lk of
@@ -856,7 +877,7 @@ delete' k0 =
       )
       l
   deleteBr l k a =
-    map
+    map'
       (error "Map.delete: L4")
       ( \rl rk ra rr _ _ ->
           case compare k0 rk of
@@ -877,7 +898,7 @@ delete' k0 =
             GT -> checkRightB l k a (deleteRr rl rk ra rr)
       )
   deleteLl l k a r =
-    map
+    map'
       (error "Map.delete: L5")
       ( \ll lk la lr _ _ ->
           case compare k0 lk of
@@ -899,7 +920,7 @@ delete' k0 =
       )
       l
   deleteLr l k a =
-    map
+    map'
       (error "Map.delete: L6")
       ( \rl rk ra rr _ _ ->
           case compare k0 rk of
@@ -920,10 +941,10 @@ delete' k0 =
             GT -> checkRightL l k a (deleteRr rl rk ra rr)
       )
   rebalanceR l k a =
-    map
+    map'
       (error "Map.delete: L7")
       ( \rl rk ra rr _ _ ->
-          map
+          map'
             (error "Map.delete: L8")
             (\rll rlk rla rlr _ _ -> B (B l k a rll) rlk rla (R rlr rk ra rr))
             (\rll rlk rla rlr _ _ -> B (B l k a rll) rlk rla (B rlr rk ra rr))
@@ -933,12 +954,12 @@ delete' k0 =
       (\rl rk ra rr _ _ -> L (R l k a rl) rk ra rr)
       (\rl rk ra rr _ _ -> B (B l k a rl) rk ra rr)
   rebalanceL l k a r =
-    map
+    map'
       (error "Map.delete: L9")
       (\ll lk la lr _ _ -> B ll lk la (B lr k a r))
       (\ll lk la lr _ _ -> R ll lk la (L lr k a r))
       ( \ll lk la lr _ _ ->
-          map
+          map'
             (error "Map.delete: L10")
             (\lrl lrk lra lrr _ _ -> B (B ll lk la lrl) lrk lra (R lrr k a r))
             (\lrl lrk lra lrr _ _ -> B (B ll lk la lrl) lrk lra (B lrr k a r))
@@ -947,49 +968,49 @@ delete' k0 =
       )
       l
   checkLeftR l k a r =
-    map
+    map'
       (error "Map.delete: L11")
       (\_ _ _ _ _ _ -> R l k a r)
       (\_ _ _ _ _ _ -> rebalanceR l k a r)
       (\_ _ _ _ _ _ -> R l k a r)
       l
   checkLeftB l k a r =
-    map
+    map'
       (error "Map.delete: L12")
       (\_ _ _ _ _ _ -> B l k a r)
       (\_ _ _ _ _ _ -> R l k a r)
       (\_ _ _ _ _ _ -> B l k a r)
       l
   checkLeftL l k a r =
-    map
+    map'
       (error "Map.delete: L13")
       (\_ _ _ _ _ _ -> L l k a r)
       (\_ _ _ _ _ _ -> B l k a r)
       (\_ _ _ _ _ _ -> L l k a r)
       l
   checkRightR l k a r =
-    map
+    map'
       (error "Map.delete: L14")
       (\_ _ _ _ _ _ -> R l k a r)
       (\_ _ _ _ _ _ -> B l k a r)
       (\_ _ _ _ _ _ -> R l k a r)
       r
   checkRightB l k a r =
-    map
+    map'
       (error "Map.delete: L15")
       (\_ _ _ _ _ _ -> B l k a r)
       (\_ _ _ _ _ _ -> L l k a r)
       (\_ _ _ _ _ _ -> B l k a r)
       r
   checkRightL l k a r =
-    map
+    map'
       (error "Map.delete: L16")
       (\_ _ _ _ _ _ -> L l k a r)
       (\_ _ _ _ _ _ -> rebalanceL l k a r)
       (\_ _ _ _ _ _ -> L l k a r)
       r
   substituteR l =
-    map
+    map'
       (error "Map.delete: L17")
       ( \rl rk ra rr _ _ ->
           (\(k, a, r) -> checkRightR l k a r) $
@@ -1004,7 +1025,7 @@ delete' k0 =
             popLeftR rl rk ra rr
       )
   substituteBr l =
-    map
+    map'
       E
       ( \rl rk ra rr _ _ ->
           (\(k, a, r) -> checkRightB l k a r) $
@@ -1019,7 +1040,7 @@ delete' k0 =
             popLeftR rl rk ra rr
       )
   substituteBl l r =
-    map
+    map'
       E
       ( \ll lk la lr _ _ ->
           (\(l', k, a) -> checkLeftB l' k a r) $
@@ -1035,7 +1056,7 @@ delete' k0 =
       )
       l
   substituteL l r =
-    map
+    map'
       (error "Map.delete: L18")
       ( \ll lk la lr _ _ ->
           (\(l', k, a) -> checkLeftL l' k a r) $
@@ -1051,49 +1072,49 @@ delete' k0 =
       )
       l
   checkLeftR' l k a r =
-    map
+    map'
       (rebalanceR l k a r)
       (\_ _ _ _ _ _ -> R l k a r)
       (\_ _ _ _ _ _ -> R l k a r)
       (\_ _ _ _ _ _ -> R l k a r)
       l
   checkLeftB' l k a r =
-    map
+    map'
       (R l k a r)
       (\_ _ _ _ _ _ -> B l k a r)
       (\_ _ _ _ _ _ -> B l k a r)
       (\_ _ _ _ _ _ -> B l k a r)
       l
   checkLeftL' l k a r =
-    map
+    map'
       (B l k a r)
       (\_ _ _ _ _ _ -> L l k a r)
       (\_ _ _ _ _ _ -> L l k a r)
       (\_ _ _ _ _ _ -> L l k a r)
       l
   checkRightR' l k a r =
-    map
+    map'
       (B l k a r)
       (\_ _ _ _ _ _ -> R l k a r)
       (\_ _ _ _ _ _ -> R l k a r)
       (\_ _ _ _ _ _ -> R l k a r)
       r
   checkRightB' l k a r =
-    map
+    map'
       (L l k a r)
       (\_ _ _ _ _ _ -> B l k a r)
       (\_ _ _ _ _ _ -> B l k a r)
       (\_ _ _ _ _ _ -> B l k a r)
       r
   checkRightL' l k a r =
-    map
+    map'
       (rebalanceL l k a r)
       (\_ _ _ _ _ _ -> L l k a r)
       (\_ _ _ _ _ _ -> L l k a r)
       (\_ _ _ _ _ _ -> L l k a r)
       r
   popLeftR l k a r =
-    map
+    map'
       (k, a, r)
       ( \ll lk la lr _ _ ->
           (\(k', a', l') -> (k', a', checkLeftR l' k a r)) $
@@ -1106,14 +1127,14 @@ delete' k0 =
       )
       l
   popLeftB l k a r =
-    map
+    map'
       (k, a, E)
       (\ll lk la lr _ _ -> popLeftBL ll lk la lr k a r)
       (\ll lk la lr _ _ -> popLeftBB ll lk la lr k a r)
       (\ll lk la lr _ _ -> popLeftBR ll lk la lr k a r)
       l
   popLeftL l k a r =
-    map
+    map'
       (error "Map.delete: L19")
       ( \ll lk la lr _ _ ->
           (\(k', a', l') -> (k', a', checkLeftL l' k a r)) $
@@ -1126,7 +1147,7 @@ delete' k0 =
       )
       l
   popLeftRB ll lk la lr k a r =
-    map
+    map'
       (lk, la, rebalanceR E k a r)
       ( \lll llk lla llr _ _ ->
           (\(k', a', l) -> (k', a', R l k a r)) $
@@ -1142,7 +1163,7 @@ delete' k0 =
       )
       ll
   popLeftBB ll lk la lr k a r =
-    map
+    map'
       (lk, la, R E k a r)
       ( \lll llk lla llr _ _ ->
           (\(k', a', l) -> (k', a', B l k a r)) $
@@ -1158,7 +1179,7 @@ delete' k0 =
       )
       ll
   popLeftLB ll lk la lr k a r =
-    map
+    map'
       (lk, la, B E k a E)
       ( \lll llk lla llr _ _ ->
           (\(k', a', l) -> (k', a', L l k a r)) $
@@ -1180,7 +1201,7 @@ delete' k0 =
     (\(k', a', l) -> (k', a', checkLeftB l k a r)) $
       popLeftL ll lk la lr
   popRightR l k a =
-    map
+    map'
       (error "Map.delete: L20")
       ( \rl rk ra rr _ _ ->
           (\(r, k', a') -> (checkRightR l k a r, k', a')) $
@@ -1192,13 +1213,13 @@ delete' k0 =
             popRightR rl rk ra rr
       )
   popRightB l k a =
-    map
+    map'
       (E, k, a)
       (\rl rk ra rr _ _ -> popRightBL l k a rl rk ra rr)
       (\rl rk ra rr _ _ -> popRightBB l k a rl rk ra rr)
       (\rl rk ra rr _ _ -> popRightBR l k a rl rk ra rr)
   popRightL l k a =
-    map
+    map'
       (l, k, a)
       ( \rl rk ra rr _ _ ->
           (\(r, k', a') -> (checkRightL l k a r, k', a')) $
@@ -1210,7 +1231,7 @@ delete' k0 =
             popRightR rl rk ra rr
       )
   popRightRB l k a rl rk ra =
-    map
+    map'
       (B E k a E, rk, ra)
       ( \rrl rrk rra rrr _ _ ->
           (\(r, k', a') -> (R l k a r, k', a')) $
@@ -1225,7 +1246,7 @@ delete' k0 =
             popRightBR rl rk ra rrl rrk rra rrr
       )
   popRightBB l k a rl rk ra =
-    map
+    map'
       (L l k a E, rk, ra)
       ( \rrl rrk rra rrr _ _ ->
           (\(r, k', a') -> (B l k a r, k', a')) $
@@ -1240,7 +1261,7 @@ delete' k0 =
             popRightBR rl rk ra rrl rrk rra rrr
       )
   popRightLB l k a rl rk ra =
-    map
+    map'
       (rebalanceL l k a E, rk, ra)
       ( \rrl rrk rra rrr _ _ ->
           (\(r, k', a') -> (L l k a r, k', a')) $
@@ -1264,7 +1285,7 @@ delete' k0 =
 -- | /O(log n)/ Insert a key and its value, combining new and old if present
 insertWithKey :: (Ord k) => (k -> a -> a -> a) -> k -> a -> Map k a -> Map k a
 insertWithKey f k0 a0 =
-  map
+  map'
     (B E k0 a0 E)
     (\l k a r _ _ -> insertL l k a r)
     (\l k a r _ _ -> insertB l k a r)
@@ -1286,12 +1307,12 @@ insertWithKey f k0 a0 =
       EQ -> L l k (f k a0 a) r
       GT -> insertLr l k a r
   insertRl l k a r =
-    map
+    map'
       (B (B E k0 a0 E) k a r)
       (\ll lk la lr _ _ -> R (insertL ll lk la lr) k a r)
       ( \ll lk la lr _ _ ->
           let l' = insertB ll lk la lr
-           in map
+           in map'
                 (error "Map.insert: L0")
                 (\_ _ _ _ _ _ -> B l' k a r)
                 (\_ _ _ _ _ _ -> R l' k a r)
@@ -1301,12 +1322,12 @@ insertWithKey f k0 a0 =
       (\ll lk la lr _ _ -> R (insertR ll lk la lr) k a r)
       l
   insertBl l k a r =
-    map
+    map'
       (L (B E k0 a0 E) k a r)
       (\ll lk la lr _ _ -> B (insertL ll lk la lr) k a r)
       ( \ll lk la lr _ _ ->
           let l' = insertB ll lk la lr
-           in map
+           in map'
                 (error "Map.insert: L1")
                 (\_ _ _ _ _ _ -> L l' k a r)
                 (\_ _ _ _ _ _ -> B l' k a r)
@@ -1316,12 +1337,12 @@ insertWithKey f k0 a0 =
       (\ll lk la lr _ _ -> B (insertR ll lk la lr) k a r)
       l
   insertBr l k a =
-    map
+    map'
       (R l k a (B E k0 a0 E))
       (\rl rk ra rr _ _ -> B l k a (insertL rl rk ra rr))
       ( \rl rk ra rr _ _ ->
           let r = insertB rl rk ra rr
-           in map
+           in map'
                 (error "Map.insert: L2")
                 (\_ _ _ _ _ _ -> R l k a r)
                 (\_ _ _ _ _ _ -> B l k a r)
@@ -1330,12 +1351,12 @@ insertWithKey f k0 a0 =
       )
       (\rl rk ra rr _ _ -> B l k a (insertR rl rk ra rr))
   insertLr l k a =
-    map
+    map'
       (B l k a (B E k0 a0 E))
       (\rl rk ra rr _ _ -> L l k a (insertL rl rk ra rr))
       ( \rl rk ra rr _ _ ->
           let r = insertB rl rk ra rr
-           in map
+           in map'
                 (error "Map.insert: L3")
                 (\_ _ _ _ _ _ -> B l k a r)
                 (\_ _ _ _ _ _ -> L l k a r)
@@ -1344,7 +1365,7 @@ insertWithKey f k0 a0 =
       )
       (\rl rk ra rr _ _ -> L l k a (insertR rl rk ra rr))
   insertRr l k a =
-    map
+    map'
       (error "Map.insert: L4")
       (\rl rk ra rr _ _ -> R l k a (insertL rl rk ra rr))
       ( \rl rk ra rr _ _ ->
@@ -1355,7 +1376,7 @@ insertWithKey f k0 a0 =
       )
       (\rl rk ra rr _ _ -> R l k a (insertR rl rk ra rr))
   insertLl l k a r =
-    map
+    map'
       (error "Map.insert: L5")
       (\ll lk la lr _ _ -> L (insertL ll lk la lr) k a r)
       ( \ll lk la lr _ _ ->
@@ -1367,12 +1388,12 @@ insertWithKey f k0 a0 =
       (\ll lk la lr _ _ -> L (insertR ll lk la lr) k a r)
       l
   insertRrr l k a rl rk ra =
-    map
+    map'
       (B (B l k a rl) rk ra (B E k0 a0 E))
       (\rrl rrk rra rrr _ _ -> R l k a (B rl rk ra (insertL rrl rrk rra rrr)))
       ( \rrl rrk rra rrr _ _ ->
           let rr = insertB rrl rrk rra rrr
-           in map
+           in map'
                 (error "Map.insert: L6")
                 (\_ _ _ _ _ _ -> B (B l k a rl) rk ra rr)
                 (\_ _ _ _ _ _ -> R l k a (B rl rk ra rr))
@@ -1381,12 +1402,12 @@ insertWithKey f k0 a0 =
       )
       (\rrl rrk rra rrr _ _ -> R l k a (B rl rk ra (insertR rrl rrk rra rrr)))
   insertLll ll lk la lr k a r =
-    map
+    map'
       (B (B E k0 a0 E) lk la (B lr k a r))
       (\lll llk lla llr _ _ -> L (B (insertL lll llk lla llr) lk la lr) k a r)
       ( \lll llk lla llr _ _ ->
           let ll' = insertB lll llk lla llr
-           in map
+           in map'
                 (error "Map.insert: L7")
                 (\_ _ _ _ _ _ -> B ll' lk la (B lr k a r))
                 (\_ _ _ _ _ _ -> L (B ll' lk la lr) k a r)
@@ -1396,12 +1417,12 @@ insertWithKey f k0 a0 =
       (\lll llk lla llr _ _ -> L (B (insertR lll llk lla llr) lk la lr) k a r)
       ll
   insertRrl l k a rl rk ra rr =
-    map
+    map'
       (B (B l k a E) k0 a0 (B E rk ra rr))
       (\rll rlk rla rlr _ _ -> R l k a (B (insertL rll rlk rla rlr) rk ra rr))
       ( \rll rlk rla rlr _ _ ->
           let rl' = insertB rll rlk rla rlr
-           in map
+           in map'
                 (error "Map.insert: L8")
                 ( \rll' rlk' rla' rlr' _ _ ->
                     B
@@ -1423,12 +1444,12 @@ insertWithKey f k0 a0 =
       (\rll rlk rla rlr _ _ -> R l k a (B (insertR rll rlk rla rlr) rk ra rr))
       rl
   insertLlr ll lk la lr k a r =
-    map
+    map'
       (B (B ll lk la E) k0 a0 (B E k a r))
       (\lrl lrk lra lrr _ _ -> L (B ll lk la (insertL lrl lrk lra lrr)) k a r)
       ( \lrl lrk lra lrr _ _ ->
           let lr' = insertB lrl lrk lra lrr
-           in map
+           in map'
                 (error "Map.insert: L9")
                 ( \lrl' lrk' lra' lrr' _ _ ->
                     B
